@@ -1,6 +1,7 @@
-from typing_extensions import Self
 import warnings
 
+from typing_extensions import Self
+
 warnings.filterwarnings("ignore")
 
 from time import sleep
@@ -9,8 +10,9 @@ import olorenchemengine as oce
 
 from .base_class import *
 
+
 class BaseDataset(BaseClass):
-    """ BaseDataset for all dataset objects
+    """BaseDataset for all dataset objects
 
     BaseDataset holds its data in a Pandas DataFrame.
 
@@ -25,18 +27,20 @@ class BaseDataset(BaseClass):
     @log_arguments
     def __init__(
         self,
-        name: str =None,
-        data: str =None,
+        name: str = None,
+        data: str = None,
         structure_col: str = None,
-        property_col: str =None,
-        feature_cols: list =[],
+        property_col: str = None,
+        feature_cols: list = [],
         date_col: str = None,
         log=True,
-        **kwargs
+        **kwargs,
     ):
 
         if name is None:
-            code = base64.urlsafe_b64encode(hashlib.md5(str(data).encode("utf-8")).digest())[:16].decode("utf-8")
+            code = base64.urlsafe_b64encode(
+                hashlib.md5(str(data).encode("utf-8")).digest()
+            )[:16].decode("utf-8")
             self.name = f"Dataset-{code}"
         else:
             self.name = name
@@ -59,51 +63,57 @@ class BaseDataset(BaseClass):
 
     @property
     def entire_dataset(self):
-        """ Returns the entire dataset
+        """Returns the entire dataset
 
         Returns:
             pd.DataFrame: The entire dataset
-            """
+        """
 
         return self.data[self.input_cols], self.data[self.property_col]
 
     @property
     def entire_dataset_split(self):
-        """ Returns a tuple of three elements where the first is the input train data,
+        """Returns a tuple of three elements where the first is the input train data,
         the second is the input validation data, and the third is the input test data
 
         Returns:
             tuple: (train_data, val_data, test_data)
         """
-        assert "split" in self.data.columns, "Dataset not split yet, please use a splitter or define the split column"
+        assert (
+            "split" in self.data.columns
+        ), "Dataset not split yet, please use a splitter or define the split column"
         return self.data["split"]
 
     @property
     def train_dataset(self):
-        """ Returns the train dataset
-        """
-        assert "split" in self.data.columns, "Dataset not split yet, please use a splitter or define the split column"
+        """Returns the train dataset"""
+        # assert "split" in list(
+        #     self.data.columns
+        # ), f"Dataset not split yet, please use a splitter or define the split column {list(self.data.columns)}"
         train = self.data[self.data["split"] == "train"]
         return train[self.input_cols], train[self.property_col]
 
     @property
     def valid_dataset(self):
-        """ Gives a tuple of two elements where the first is the input val data and
+        """Gives a tuple of two elements where the first is the input val data and
         the second is the property of interest
 
         Returns:
             pd.DataFrame: The validation data
 
         """
-        assert "split" in self.data.columns, "Dataset not split yet, please use a splitter or define the split column"
+        assert (
+            "split" in self.data.columns
+        ), "Dataset not split yet, please use a splitter or define the split column"
         valid = self.data[self.data["split"] == "valid"]
         return valid[self.input_cols], valid[self.property_col]
 
     @property
     def trainval_dataset(self):
-        """ Returns the train and validation dataset
-        """
-        assert "split" in self.data.columns, "Dataset not split yet, please use a splitter or define the split column"
+        """Returns the train and validation dataset"""
+        assert (
+            "split" in self.data.columns
+        ), "Dataset not split yet, please use a splitter or define the split column"
         train = self.data[self.data["split"] == "train"]
         val = self.data[self.data["split"] == "valid"]
         trainval = pd.concat([train, val])
@@ -111,23 +121,29 @@ class BaseDataset(BaseClass):
 
     @property
     def test_dataset(self):
-        """ Gives a tuple of two elements where the first is the input test data and
+        """Gives a tuple of two elements where the first is the input test data and
         the second is the property of interest
 
         Returns:
             pd.DataFrame: The test data
 
         """
-        assert "split" in self.data.columns, "Dataset not split yet, please use a splitter or define the split column"
+        assert (
+            "split" in self.data.columns
+        ), "Dataset not split yet, please use a splitter or define the split column"
         test = self.data[self.data["split"] == "test"]
         return test[self.input_cols], test[self.property_col]
 
     @property
     def size(self):
-        return (len(self.train_dataset[0]), len(self.valid_dataset[0]), len(self.test_dataset[0]))
+        return (
+            len(self.train_dataset[0]),
+            len(self.valid_dataset[0]),
+            len(self.test_dataset[0]),
+        )
 
     def transform(self, dataset: Self):
-        """ Combines this dataset with the passed dataset object"""
+        """Combines this dataset with the passed dataset object"""
         # Rename passed dataset to homogenize with this dataset
 
     def _save(self):
@@ -139,8 +155,9 @@ class BaseDataset(BaseClass):
 
         self.data = pd.read_csv(StringIO(d["data_save"]))
 
+
 class BaseDatasetTransform(BaseClass):
-    """ Applies a transformation onto the inputted BaseDataset.
+    """Applies a transformation onto the inputted BaseDataset.
 
     Transformation applied as defined in the abstract method transform.
 
@@ -150,7 +167,7 @@ class BaseDatasetTransform(BaseClass):
 
     @abstractmethod
     def transform(self, dataset: BaseDataset) -> BaseDataset:
-        """ Applies a transformation onto the inputted BaseDataset.
+        """Applies a transformation onto the inputted BaseDataset.
 
         Parameters:
         dataset (BaseDataset): The dataset to transform.
@@ -163,13 +180,16 @@ class BaseDatasetTransform(BaseClass):
     def _load(self, d):
         pass
 
+
 def func(self: BaseDataset, other: BaseDatasetTransform) -> BaseDataset:
     return other.transform(self)
 
+
 BaseDataset.__add__ = func
 
+
 class DatasetFromCSV(BaseDataset):
-    """ DatasetFromFile for all dataset objects
+    """DatasetFromFile for all dataset objects
 
     Parameters:
         file_path (str): Relative or absolute to a local CSV file
@@ -182,12 +202,13 @@ class DatasetFromCSV(BaseDataset):
         super().__init__(data=self.data.to_csv(), log=False, **kwargs)
 
 
-import requests
 import io
 
+import requests
+
 
 class DatasetFromCDDSearch(BaseDataset):
-    """ Dataset for retreiving data from CDD via a saved search.
+    """Dataset for retreiving data from CDD via a saved search.
 
     Requires a CDD Token to be set.
 
@@ -197,7 +218,9 @@ class DatasetFromCDDSearch(BaseDataset):
         update (bool): Whether or no to update the cached dataset by redoing the CDD search"""
 
     @log_arguments
-    def __init__(self, search_id, cache_file_path=None, update=True, log=True, **kwargs):
+    def __init__(
+        self, search_id, cache_file_path=None, update=True, log=True, **kwargs
+    ):
 
         if cache_file_path is None:
             cache_file_path = f"{search_id}.csv"
@@ -212,13 +235,15 @@ class DatasetFromCDDSearch(BaseDataset):
         super().__init__(data=self.data.to_csv(), log=False, **kwargs)
 
     def run_saved_search(self, search_id):
-        """  Uses a CDD Token (passed as search_id) to search saved datasets to find and return its
+        """Uses a CDD Token (passed as search_id) to search saved datasets to find and return its
         related dataset export id.
 
         Parameters:
         search_id (str): The ID of the saved CDD search to use.
         """
-        base_url = f"https://app.collaborativedrug.com/api/v1/vaults/{oce.CONFIG['VAULT_ID']}/"
+        base_url = (
+            f"https://app.collaborativedrug.com/api/v1/vaults/{oce.CONFIG['VAULT_ID']}/"
+        )
         headers = {"X-CDD-token": f"{oce.CONFIG['CDD_TOKEN']}"}
         url = base_url + f"searches/{search_id}"
 
@@ -233,7 +258,9 @@ class DatasetFromCDDSearch(BaseDataset):
         export_id (str): The unique export ID of the dataset searched for
 
         """
-        base_url = f"https://app.collaborativedrug.com/api/v1/vaults/{oce.CONFIG['VAULT_ID']}/"
+        base_url = (
+            f"https://app.collaborativedrug.com/api/v1/vaults/{oce.CONFIG['VAULT_ID']}/"
+        )
         headers = {"X-CDD-token": f"{oce.CONFIG['CDD_TOKEN']}"}
         url = base_url + f"export_progress/{export_id}"
 
@@ -248,7 +275,9 @@ class DatasetFromCDDSearch(BaseDataset):
         export_id (str): The unique export ID of the dataset searched for
 
         """
-        base_url = f"https://app.collaborativedrug.com/api/v1/vaults/{oce.CONFIG['VAULT_ID']}/"
+        base_url = (
+            f"https://app.collaborativedrug.com/api/v1/vaults/{oce.CONFIG['VAULT_ID']}/"
+        )
         headers = {"X-CDD-token": f"{oce.CONFIG['CDD_TOKEN']}"}
         url = base_url + f"exports/{export_id}"
 
@@ -257,7 +286,7 @@ class DatasetFromCDDSearch(BaseDataset):
         return pd.read_csv(data_stream)
 
     def get_dataset_cdd_saved_search(self, search_id):
-        """ Uses a CDD Token (passed as search_id) to search saved datasets to find and return its
+        """Uses a CDD Token (passed as search_id) to search saved datasets to find and return its
         related dataset export id. Using the export id, it then checks the export status and
         returns the dataset's data in CSV format.
 
@@ -269,7 +298,7 @@ class DatasetFromCDDSearch(BaseDataset):
         status = "new"
         while True:
             print(f"Export status is {status}, checking in {2**i} seconds...")
-            sleep(2 ** i)
+            sleep(2**i)
             status = self.check_export_status(export_id)
             if status == "finished":
                 print("Export ready!")
@@ -277,10 +306,12 @@ class DatasetFromCDDSearch(BaseDataset):
             i += 1
         return self.get_export(export_id)
 
+
 from rdkit import Chem
 
+
 class CleanStructures(BaseDatasetTransform):
-    """ CleanStructures creates a new dataset from the original dataset by removing
+    """CleanStructures creates a new dataset from the original dataset by removing
     structures that are not valid.
 
     Parameters:
@@ -289,13 +320,17 @@ class CleanStructures(BaseDatasetTransform):
 
     def transform(self, dataset: BaseDataset, dropna_property: bool = True, **kwargs):
         cols = dataset.data.columns.tolist()
+
         def try_clean(s):
             try:
                 return Chem.MolToSmiles(Chem.MolFromSmiles(s))
             except:
                 return None
+
         start_num = len(dataset.data)
-        dataset.data[dataset.structure_col] = dataset.data[dataset.structure_col].apply(lambda x: try_clean(x))
+        dataset.data[dataset.structure_col] = dataset.data[dataset.structure_col].apply(
+            lambda x: try_clean(x)
+        )
         dataset.data = dataset.data.dropna(subset=[dataset.structure_col])
         if hasattr(dataset, "property_col") and not dataset.property_col is None:
             dataset.data = dataset.data.dropna(subset=[dataset.property_col])
@@ -303,8 +338,9 @@ class CleanStructures(BaseDatasetTransform):
         print(f"{start_num - len(dataset.data)} structure(s) were removed.")
         return dataset
 
+
 class Discretize(BaseDatasetTransform):
-    """ Discretize creates a new dataset from the original dataset by discretizing
+    """Discretize creates a new dataset from the original dataset by discretizing
     the property column.
 
     Parameters:
@@ -318,11 +354,14 @@ class Discretize(BaseDatasetTransform):
         self.prop_cutoff = prop_cutoff
 
     def transform(self, dataset: BaseDataset, **kwargs):
-        dataset.data[dataset.property_col] = [1 if x > self.prop_cutoff else 0 for x in dataset.data[dataset.property_col]]
+        dataset.data[dataset.property_col] = [
+            1 if x > self.prop_cutoff else 0 for x in dataset.data[dataset.property_col]
+        ]
         return dataset
 
+
 class OneHotEncode(BaseDatasetTransform):
-    """ This one hot encodes a given feature column
+    """This one hot encodes a given feature column
 
     Parameters:
         feature_col (str): The feature column to one hot encode.
@@ -336,4 +375,4 @@ class OneHotEncode(BaseDatasetTransform):
         one_hot = pd.get_dummies(dataset.data[self.feature_col])
         dataset.data = dataset.data.join(one_hot)
         dataset.feature_cols += one_hot.columns.tolist()
-        return dataset
\ No newline at end of file
+        return dataset
