@@ -199,10 +199,11 @@ def download_public_file(path, redownload=False):
         os.makedirs(os.path.dirname(local_path))
 
     print(f"Downloading {path}...")
+    import urllib
     import urllib.request
 
     urllib.request.urlretrieve(
-        f"https://storage.googleapis.com/oloren-public-data/{path}", local_path
+        urllib.parse.quote(f"https://storage.googleapis.com/oloren-public-data/{path}",safe='/:?=&'), local_path
     )
     return local_path
 
@@ -561,7 +562,10 @@ class BaseRemoteSymbol:
         return iter(_runtime.get_iterable(self.REMOTE_ID))
 
     def __repr__(self):
-        return _runtime.get_obj_repr(self.REMOTE_ID)
+        if hasattr(self, "REMOTE_ID"):
+            return _runtime.get_obj_repr(self.REMOTE_ID)
+        else:
+            return  object.__repr__(self)
 
     def _upload_remote(self):
         try:
@@ -609,8 +613,6 @@ class BaseRemoteSymbol:
         return self.REMOTE_CHILDREN[key]
 
     def __call__(self, *args, **kwargs):
-        # TODO: move logic from init to here, then return baseremotesymbol only if return type is none
-
         remote_id = generate_uuid()
         out = _runtime.add_instruction(
             {
@@ -905,7 +907,7 @@ def save(model: BaseClass, fname: str):
 
         REMOTE_ID = model.REMOTE_ID
         oas_connector.storage.child(
-            f"{oas_connector.uid}/sessions/{_runtime.session_id}/{REMOTE_ID}.oce"
+            f"{oas_connector.uid}/sessions/{_runtime.session_id}/{REMOTE_ID}.oce.delete"
         ).download("", fname, token=oas_connector.authenticate())
     else:
         save_dict = saves(model)
