@@ -1,26 +1,20 @@
 import copy
 import io
-
 from abc import abstractmethod
-from tqdm import tqdm
 
 import numpy as np
+import PIL
+import selfies as sf
+from rdkit import Chem
+from rdkit.Chem import AllChem
+from rdkit.DataStructs.cDataStructs import BulkTanimotoSimilarity, TanimotoSimilarity
+from tqdm import tqdm
 
 from .base_class import *
 from .dataset import *
-from .representations import *
 from .external.stoned import *
+from .representations import *
 
-import PIL
-
-try:
-    import selfies as sf
-except ImportError:
-    mock_imports(globals(), "sf")
-
-from rdkit.DataStructs.cDataStructs import BulkTanimotoSimilarity, TanimotoSimilarity
-from rdkit import Chem
-from rdkit.Chem import AllChem
 
 class PerturbationEngine(BaseClass):
 
@@ -47,6 +41,7 @@ class PerturbationEngine(BaseClass):
     def get_compound_list(self, smiles, **kwargs) -> list:
         pass
 
+
 class SwapMutations(PerturbationEngine):
     """SwapMutations replaces substructures with radius r with another substructure
     with radius < r. The substructure is chosen such that it has the same outgoing
@@ -60,72 +55,137 @@ class SwapMutations(PerturbationEngine):
             entire class of modifications"""
 
     @log_arguments
-    def __init__(self, radius = 0, log = True):
+    def __init__(self, radius=0, log=True):
         self.radius = radius
 
         if not radius in (0, 1, 2):
             raise ValueError("radius must be 0, 1, or 2")
 
-
         self.trans = dict()
-        for radius in range(radius+1):
-            transformation_path = download_public_file(f"swap-mutations/trans_{radius}.json")
+        for radius in range(radius + 1):
+            transformation_path = download_public_file(
+                f"swap-mutations/trans_{radius}.json"
+            )
             with open(transformation_path, "r") as f:
                 self.trans.update(json.load(f))
 
-    def get_substitution(self, m, idx, r = 1):
+    def get_substitution(self, m, idx, r=1):
 
         # finding atoms within radius r of atom idx
         substructure_atoms = set({idx})
         for i in range(r):
-            bonds = [b for a_ in substructure_atoms for b in m.GetAtomWithIdx(a_).GetBonds()]
-            substructure_atoms_ = {b.GetBeginAtomIdx() for b in bonds}.union({b.GetEndAtomIdx() for b in bonds})
+            bonds = [
+                b for a_ in substructure_atoms for b in m.GetAtomWithIdx(a_).GetBonds()
+            ]
+            substructure_atoms_ = {b.GetBeginAtomIdx() for b in bonds}.union(
+                {b.GetEndAtomIdx() for b in bonds}
+            )
             substructure_atoms = substructure_atoms.union(substructure_atoms_)
 
         # finding atoms connecting to substructure
-        surface_atoms = {b.GetEndAtomIdx() for a_ in substructure_atoms for b in m.GetAtomWithIdx(a_).GetBonds() if not b.GetEndAtomIdx() in substructure_atoms}
-        surface_atoms.update({b.GetBeginAtomIdx() for a_ in substructure_atoms for b in m.GetAtomWithIdx(a_).GetBonds() if not b.GetBeginAtomIdx() in substructure_atoms})
+        surface_atoms = {
+            b.GetEndAtomIdx()
+            for a_ in substructure_atoms
+            for b in m.GetAtomWithIdx(a_).GetBonds()
+            if not b.GetEndAtomIdx() in substructure_atoms
+        }
+        surface_atoms.update(
+            {
+                b.GetBeginAtomIdx()
+                for a_ in substructure_atoms
+                for b in m.GetAtomWithIdx(a_).GetBonds()
+                if not b.GetBeginAtomIdx() in substructure_atoms
+            }
+        )
         surface_atoms = list(surface_atoms)
         surface_l = list()
         for surface_atom in surface_atoms:
-            bonds = [b.GetIdx() for b in m.GetAtomWithIdx(surface_atom).GetBonds() if b.GetEndAtomIdx() in substructure_atoms or b.GetBeginAtomIdx() in substructure_atoms]
-            surface_l.append((m.GetAtomWithIdx(surface_atom).GetAtomicNum(),
-                              [m.GetBondWithIdx(b).GetBondTypeAsDouble() for b in bonds]))
+            bonds = [
+                b.GetIdx()
+                for b in m.GetAtomWithIdx(surface_atom).GetBonds()
+                if b.GetEndAtomIdx() in substructure_atoms
+                or b.GetBeginAtomIdx() in substructure_atoms
+            ]
+            surface_l.append(
+                (
+                    m.GetAtomWithIdx(surface_atom).GetAtomicNum(),
+                    [m.GetBondWithIdx(b).GetBondTypeAsDouble() for b in bonds],
+                )
+            )
 
-        tmp = [(x1, x2, x3) for x1, x2, x3 in zip([sum(surface_b) for surface_a, surface_b in surface_l], surface_atoms, surface_l)]
-        tmp.sort(key = lambda x: x[0])
+        tmp = [
+            (x1, x2, x3)
+            for x1, x2, x3 in zip(
+                [sum(surface_b) for surface_a, surface_b in surface_l],
+                surface_atoms,
+                surface_l,
+            )
+        ]
+        tmp.sort(key=lambda x: x[0])
         surface_l = [x for _, _, x in tmp]
         surface_atoms = [x for _, x, _ in tmp]
 
         return surface_l, surface_atoms, substructure_atoms
 
-    def get_entry(self, m, idx, r = 1):
+    def get_entry(self, m, idx, r=1):
         m = copy.deepcopy(m)
 
         # finding atoms within radius r of atom idx
         substructure_atoms = set({idx})
         for i in range(r):
-            bonds = [b for a_ in substructure_atoms for b in m.GetAtomWithIdx(a_).GetBonds()]
-            substructure_atoms_ = {b.GetBeginAtomIdx() for b in bonds}.union({b.GetEndAtomIdx() for b in bonds})
+            bonds = [
+                b for a_ in substructure_atoms for b in m.GetAtomWithIdx(a_).GetBonds()
+            ]
+            substructure_atoms_ = {b.GetBeginAtomIdx() for b in bonds}.union(
+                {b.GetEndAtomIdx() for b in bonds}
+            )
             substructure_atoms = substructure_atoms.union(substructure_atoms_)
 
         # finding atoms connecting to substructure
-        surface_atoms = {b.GetEndAtomIdx() for a_ in substructure_atoms for b in m.GetAtomWithIdx(a_).GetBonds() if not b.GetEndAtomIdx() in substructure_atoms}
-        surface_atoms.update({b.GetBeginAtomIdx() for a_ in substructure_atoms for b in m.GetAtomWithIdx(a_).GetBonds() if not b.GetBeginAtomIdx() in substructure_atoms})
+        surface_atoms = {
+            b.GetEndAtomIdx()
+            for a_ in substructure_atoms
+            for b in m.GetAtomWithIdx(a_).GetBonds()
+            if not b.GetEndAtomIdx() in substructure_atoms
+        }
+        surface_atoms.update(
+            {
+                b.GetBeginAtomIdx()
+                for a_ in substructure_atoms
+                for b in m.GetAtomWithIdx(a_).GetBonds()
+                if not b.GetBeginAtomIdx() in substructure_atoms
+            }
+        )
         surface_atoms = list(surface_atoms)
         surface_l = list()
         for surface_atom in surface_atoms:
-            bonds = [b.GetIdx() for b in m.GetAtomWithIdx(surface_atom).GetBonds() if b.GetEndAtomIdx() in substructure_atoms or b.GetBeginAtomIdx() in substructure_atoms]
-            surface_l.append((m.GetAtomWithIdx(surface_atom).GetAtomicNum(),
-                              [m.GetBondWithIdx(b).GetBondTypeAsDouble() for b in bonds]))
+            bonds = [
+                b.GetIdx()
+                for b in m.GetAtomWithIdx(surface_atom).GetBonds()
+                if b.GetEndAtomIdx() in substructure_atoms
+                or b.GetBeginAtomIdx() in substructure_atoms
+            ]
+            surface_l.append(
+                (
+                    m.GetAtomWithIdx(surface_atom).GetAtomicNum(),
+                    [m.GetBondWithIdx(b).GetBondTypeAsDouble() for b in bonds],
+                )
+            )
 
-        tmp = [(x1, x2, x3) for x1, x2, x3 in zip([sum(surface_b) for surface_a, surface_b in surface_l], surface_atoms, surface_l)]
-        tmp.sort(key = lambda x: x[0])
+        tmp = [
+            (x1, x2, x3)
+            for x1, x2, x3 in zip(
+                [sum(surface_b) for surface_a, surface_b in surface_l],
+                surface_atoms,
+                surface_l,
+            )
+        ]
+        tmp.sort(key=lambda x: x[0])
         surface_l = [x for _, _, x in tmp]
         surface_atoms = [x for _, x, _ in tmp]
 
         for i, j in enumerate(surface_atoms):
-            m.GetAtomWithIdx(j).SetAtomMapNum(i+1)
+            m.GetAtomWithIdx(j).SetAtomMapNum(i + 1)
 
         substruct = Chem.RWMol(m)
 
@@ -164,7 +224,6 @@ class SwapMutations(PerturbationEngine):
 
                 removal.append(v_)
 
-
         for remove in removal:
             m.RemoveAtom(remove.GetIdx())
         return m
@@ -172,7 +231,7 @@ class SwapMutations(PerturbationEngine):
     def get_compound_at_idx(self, mol, idx, **kwargs):
         ref_m = mol
 
-        l, a, sub_a = self.get_substitution(ref_m, idx, r = self.radius)
+        l, a, sub_a = self.get_substitution(ref_m, idx, r=self.radius)
         # if the current substructure to replace has no replacements in the database
         if str(l) not in self.trans:
             return None
@@ -181,7 +240,7 @@ class SwapMutations(PerturbationEngine):
             ref_m.GetAtomWithIdx(x).SetAtomMapNum(10000)
 
         for k, j in enumerate(a):
-            ref_m.GetAtomWithIdx(j).SetAtomMapNum(k+1)
+            ref_m.GetAtomWithIdx(j).SetAtomMapNum(k + 1)
 
         sub = np.random.choice(self.trans[str(l)])
         m = Chem.CombineMols(ref_m, Chem.MolFromSmiles(sub, sanitize=False))
@@ -196,7 +255,7 @@ class SwapMutations(PerturbationEngine):
         for remove in removal:
             m.RemoveAtom(remove.GetIdx())
         if m is None:
-            errors+=1
+            errors += 1
         try:
             s1 = Chem.MolToSmiles(m)
             m = self.stitch(m)
@@ -211,7 +270,7 @@ class SwapMutations(PerturbationEngine):
             smiles = Chem.MolToSmiles(mol)
         else:
             mol = Chem.MolFromSmiles(smiles)
-        idx = np.random.choice(mol.GetNumAtoms(), replace = False)
+        idx = np.random.choice(mol.GetNumAtoms(), replace=False)
         return self.get_compound_at_idx(mol, idx, **kwargs)
 
     def get_compound_list(self, smiles, idx: int = None, **kwargs) -> list:
@@ -223,13 +282,13 @@ class SwapMutations(PerturbationEngine):
             space = [idx]
         for i in tqdm(space):
             ref_m = Chem.MolFromSmiles(smiles)
-            l, a, sub_a = self.get_substitution(ref_m, i, r= self.radius)
+            l, a, sub_a = self.get_substitution(ref_m, i, r=self.radius)
 
             for x in sub_a:
                 ref_m.GetAtomWithIdx(x).SetAtomMapNum(10000)
 
             for k, j in enumerate(a):
-                ref_m.GetAtomWithIdx(j).SetAtomMapNum(k+1)
+                ref_m.GetAtomWithIdx(j).SetAtomMapNum(k + 1)
 
             surface_atoms = [ref_m.GetAtomWithIdx(j) for j in a]
             if str(l) not in self.trans.keys():
@@ -263,6 +322,7 @@ class SwapMutations(PerturbationEngine):
     def _load(self, d: dict):
         return super()._load(d)
 
+
 class STONEDMutations(PerturbationEngine):
     """Implements STONED-SELFIES algorithm for generating modified compounds.
 
@@ -277,7 +337,7 @@ class STONEDMutations(PerturbationEngine):
     """
 
     @log_arguments
-    def __init__(self, mutations: int = 1, log = True):
+    def __init__(self, mutations: int = 1, log=True):
         self.mutations = mutations
 
     def get_compound_at_idx(self, mol: Chem.Mol, idx: int, **kwargs) -> str:
@@ -285,7 +345,7 @@ class STONEDMutations(PerturbationEngine):
         selfie = sf.encoder(smiles)
         selfie_chars = get_selfie_chars(selfie)
         max_molecules_len = len(selfie_chars) + self.mutations
-        selfie_mutated, _ = mutate_selfie(selfie, max_molecules_len, index = idx)
+        selfie_mutated, _ = mutate_selfie(selfie, max_molecules_len, index=idx)
         smiles_back = sf.decoder(selfie_mutated)
         return smiles_back
 
@@ -293,15 +353,19 @@ class STONEDMutations(PerturbationEngine):
         ref_m = Chem.MolFromSmiles(smiles)
         randomized_smiles_ordering = randomize_smiles(ref_m)
         selfie = sf.encoder(randomized_smiles_ordering)
-        selfie_mutated = get_mutated_SELFIES([selfie], num_mutations = self.mutations)[0]
+        selfie_mutated = get_mutated_SELFIES([selfie], num_mutations=self.mutations)[0]
         smiles_back = sf.decoder(selfie_mutated)
         return smiles_back
 
     def get_compound_list(self, smiles: str, num_samples: int = 1000, **kwargs) -> list:
         ref_m = Chem.MolFromSmiles(smiles)
-        randomized_smile_orderings = [randomize_smiles(ref_m) for _ in range(num_samples)]
+        randomized_smile_orderings = [
+            randomize_smiles(ref_m) for _ in range(num_samples)
+        ]
         selfies_ls = [sf.encoder(x) for x in randomized_smile_orderings]
-        selfies_mut = get_mutated_SELFIES(selfies_ls.copy(), num_mutations = self.mutations)
+        selfies_mut = get_mutated_SELFIES(
+            selfies_ls.copy(), num_mutations=self.mutations
+        )
         smiles_back = [sf.decoder(x) for x in selfies_mut]
         return list(set(smiles_back))
 
@@ -311,6 +375,7 @@ class STONEDMutations(PerturbationEngine):
     def _load(self, d: dict):
         return super()._load(d)
 
+
 class CounterfactualEngine(BaseClass):
     """Generates counterfactual compounds based on:
 
@@ -319,7 +384,9 @@ class CounterfactualEngine(BaseClass):
     """
 
     @log_arguments
-    def __init__(self, model: BaseModel, perturbation_engine: PerturbationEngine = "default"):
+    def __init__(
+        self, model: BaseModel, perturbation_engine: PerturbationEngine = "default"
+    ):
         self.model = model
         if perturbation_engine == "default":
             self.perturbation_engine = SwapMutations(radius=1)
@@ -356,18 +423,19 @@ class CounterfactualEngine(BaseClass):
 
         from sklearn.cluster import DBSCAN
         from sklearn.decomposition import PCA
-        coords = PCA(n_components = 2).fit_transform(dmat)
+
+        coords = PCA(n_components=2).fit_transform(dmat)
         clustering = DBSCAN(eps=0.15, min_samples=5).fit(coords)
 
         samples = {
-            'SMILES': filtered_smiles,
-            'Similarity': scores,
-            'Value': values,
-            'Coordinate': list(coords),
-            'Cluster': clustering.labels_,
-            'Original': [True] + [False for _ in filtered_smiles[1:]]
+            "SMILES": filtered_smiles,
+            "Similarity": scores,
+            "Value": values,
+            "Coordinate": list(coords),
+            "Cluster": clustering.labels_,
+            "Original": [True] + [False for _ in filtered_smiles[1:]],
         }
-        self.samples = pd.DataFrame(samples).to_dict('records')
+        self.samples = pd.DataFrame(samples).to_dict("records")
 
     def get_samples(self) -> pd.DataFrame:
         """Returns candidate counterfactuals as a pandas dataframe.
@@ -377,23 +445,25 @@ class CounterfactualEngine(BaseClass):
         """
         return pd.DataFrame(self.samples)
 
-    def generate_cfs(self, delta: Union[int, float, Tuple] = (-1, 1), n: int = 4) -> None:
+    def generate_cfs(
+        self, delta: Union[int, float, Tuple] = (-1, 1), n: int = 4
+    ) -> None:
         """Generates counterfactuals and stores them in `self.cfs` as a list of dictionaries.
 
         Args:
             delta: margin defining counterfactuals for regression models
             n: number of counterfactuals
         """
-        base = self.samples[0]['Value']
+        base = self.samples[0]["Value"]
 
         if self.model.setting == "classification":
-            cfs = self._select_cfs(lambda s: s['Value'] != base, n)
+            cfs = self._select_cfs(lambda s: s["Value"] != base, n)
             self.cfs = self.samples[:1] + cfs
         else:
             if type(delta) in (float, int):
                 delta = (-delta, delta)
-            lcfs = self._select_cfs(lambda s: s['Value'] - delta[0] < base, n // 2)
-            hcfs = self._select_cfs(lambda s: s['Value'] - delta[1] > base, n // 2)
+            lcfs = self._select_cfs(lambda s: s["Value"] - delta[0] < base, n // 2)
+            hcfs = self._select_cfs(lambda s: s["Value"] - delta[1] > base, n // 2)
             self.cfs = self.samples[:1] + lcfs + hcfs
 
     def get_cfs(self) -> pd.DataFrame:
@@ -425,15 +495,15 @@ class CounterfactualEngine(BaseClass):
             Returns:
                 score: score for clustering
             """
-            return (s['Cluster'] == i) * fn(s) * s['Similarity']
+            return (s["Cluster"] == i) * fn(s) * s["Similarity"]
 
-        clusters = {s['Cluster'] for s in self.samples[1:]}
+        clusters = {s["Cluster"] for s in self.samples[1:]}
         for i in clusters:
             candidate = max(self.samples[1:], key=lambda s: cluster_score(s, i))
             if cluster_score(candidate, i):
                 cfs.append(candidate)
 
-        cfs = sorted(cfs, key=lambda s: s['Similarity'], reverse=True)[:n]
+        cfs = sorted(cfs, key=lambda s: s["Similarity"], reverse=True)[:n]
         return cfs
 
     def _save(self) -> dict:
@@ -442,7 +512,13 @@ class CounterfactualEngine(BaseClass):
     def _load(self, d: dict):
         return super()._load(d)
 
-def model_molecule_sensitivity(model: BaseModel, smiles: str, perturbation_engine: PerturbationEngine = "default", n: int = 30) -> Chem.Mol:
+
+def model_molecule_sensitivity(
+    model: BaseModel,
+    smiles: str,
+    perturbation_engine: PerturbationEngine = "default",
+    n: int = 30,
+) -> Chem.Mol:
 
     """
     Calculates the sensitivity of a model to perturbations in on each of a molecule's atoms,
@@ -484,4 +560,4 @@ def model_molecule_sensitivity(model: BaseModel, smiles: str, perturbation_engin
         v = np.std(predictions)
         a.SetProp("sensitivity", str(v))
         a.SetProp("atomNote", str(np.format_float_scientific(v, precision=2)))
-    return mol
\ No newline at end of file
+    return mol
